@@ -1,8 +1,12 @@
-broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE){
+broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE, isV=FALSE, .vcov=NULL, ...){
 #ogg: l'oggetto segmented
 #term: una lista *nominata* con i valori rispetto a cui calcolare i fitted
 #   OPPURE una stringa per indicare la variabile segmented OPPURE NULL (se c'e' solo una variabile)
-  dummy.matrix<-function(x.values, x.name, obj.seg, psi.est=TRUE){
+#is: 2 valori T/F per indicare se le variabili U e V nella matrice X, andrebbero sostituite con le versioni ind-smooth prima di calcolare var(X\hat\beta)
+#...: argomenti da passare a vcov.segmented(): per esempio var.diff, is, p.df  
+    if(length(isV)==1) isV<-c(FALSE,isV)
+    dummy.matrix<-NULL
+    dummy.matrix<-function(x.values, x.name, obj.seg, psi.est=TRUE, isV=FALSE){
     #given the segmented fit 'obj.seg' and a segmented variable x.name with corresponding values x.values,
     #this function simply returns a matrix with columns (x, (x-psi)_+, -b*I(x>psi))
     #or  ((x-psi)_+, -b*I(x>psi)) if obj.seg does not include the coef for the linear "x"
@@ -20,6 +24,7 @@ broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE){
           if(!is.null(term)) nomiU.ok<-(1:k)[nomiU.ok%in%term]
           return(nomiU.ok)
         }
+        if(length(isV)==1) isV<-c(FALSE,isV)
         n<-length(x.values)
         #le seguenti righe selezionavano (ERRONEAMENTE) sia "U1.x" sia "U1.neg.x" (se "x" e "neg.x" erano segmented covariates)
         #nameU<- grep(paste("\\.",x.name,"$", sep=""), obj.seg$nameUV$U, value = TRUE)
@@ -29,16 +34,18 @@ broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE){
 
         diffSlope<-coef(obj.seg)[nameU]
         est.psi<-obj.seg$psi[nameV, "Est."]
-
+        se.psi<-obj.seg$psi[nameV, "St.Err"]
+        if(any(is.na(se.psi))) stop("The St.Err. of psi is NA", call. = FALSE)
         k<-length(est.psi)
 
         PSI <- matrix(rep(est.psi, rep(n, k)), ncol = k)
+        SE.PSI <- matrix(rep(se.psi, rep(n, k)), ncol = k)
         newZ<-matrix(x.values, nrow=n,ncol=k, byrow = FALSE)
 
-        dummy1<-pmax(newZ-PSI,0)
+        dummy1<-if(isV[1]) (newZ-PSI)*pnorm((newZ-PSI)/SE.PSI) else (newZ-PSI)*(newZ>PSI) #pmax(newZ-PSI,0)
         
         if(psi.est){
-          V<-ifelse(newZ>PSI,-1,0)
+          V<-if(isV[2]) -pnorm((newZ-PSI)/SE.PSI) else -(newZ>PSI) #ifelse(newZ>PSI,-1,0)
           dummy2<- if(k==1) V*diffSlope  else V%*%diag(diffSlope) #t(diffSlope*t(-I(newZ>PSI)))
           newd<-cbind(x.values,dummy1,dummy2)
           colnames(newd)<-c(x.name,nameU, nameV)
@@ -50,7 +57,7 @@ broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE){
         return(newd)
     }
 #--------------
-        f.U<-function(nomiU, term=NULL){
+    f.U<-function(nomiU, term=NULL){
         #trasforma i nomi dei coeff U (o V) nei nomi delle variabili corrispondenti
         #and if 'term' is provided (i.e. it differs from NULL) the index of nomiU matching term are returned
             k<-length(nomiU)
@@ -65,6 +72,12 @@ broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE){
           return(nomiU.ok)
         }
 #-------------
+    if(se.fit) {
+      if(is.null(.vcov)) .vcov<-vcov.segmented(ogg, ...)
+      #---Dalla versione 1.2.0 (20/06/20) ho eliminato il controllo sotto per consentire l'utilizzo
+      #----di modelli che restituivano una cov con dimensione diversa dal numero dei coeff lineari (ad es., censReg)
+      #if(!all(dim(.vcov)==c(length(ogg$coef), length(ogg$coef)))) stop("Incorrect dimension of cov matrix", call. = FALSE)
+      }
     xvalues<-term
     nomeV <- ogg$nameUV$V
     nomeU <- ogg$nameUV$U
@@ -100,24 +113,27 @@ broken.line<-function(ogg, term=NULL, link=TRUE, interc=TRUE, se.fit=TRUE){
             #grep(paste("\\.", nomeZ[i], "$", sep = ""), nomiSenzaU, value = FALSE))
             }
     ste.fit<-fit <- vector(mode = "list", length = length(nomeZ))
-    for (i in 1:n.seg) {
+        for (i in 1:n.seg) {
         x.name <- nomeZ[i]
-        X<-dummy.matrix(unlist(xvalues), x.name, ogg)#<--NB: xvalues non varia con i!!! perche' farlo calcolare comunque? 
+        Xfit<-dummy.matrix(unlist(xvalues), x.name, ogg, isV=FALSE)
+        if(se.fit) X<-dummy.matrix(unlist(xvalues), x.name, ogg, isV=isV)#<--NB: xvalues non varia con i!!! perche' farlo calcolare comunque? 
         ind <- as.numeric(na.omit(unlist(index[[i]])))
         if(interc && "(Intercept)"%in%nomi) {
           ind<- c(match("(Intercept)",nomi),ind)
-          X<-cbind(1,X)
+          Xfit<-cbind(1,Xfit)
+          if(se.fit) X<-cbind(1,X)
           }
         cof <- coef(ogg)[ind]
-        fit[[i]]<-drop(X%*%cof)
-        ste.fit[[i]] <- if(!se.fit) 10 else sqrt(rowSums((X %*% vcov(ogg)[ind,ind]) * X)) #sqrt(diag(X%*%Var%*%t(X)))
+        fit[[i]]<-drop(Xfit%*%cof)
+        if(se.fit) ste.fit[[i]] <- sqrt(rowSums((X %*% .vcov[ind,ind]) * X)) #sqrt(diag(X%*%Var%*%t(X)))
+        #ste.fit[[i]] <- if(!se.fit) 10 else sqrt(rowSums((X %*% vcov.segmented(ogg,...)[ind,ind]) * X)) #sqrt(diag(X%*%Var%*%t(X)))
         }
         names(fit)<- names(ste.fit)<- nomeZ
         r<-list(fit=fit[[nomeOK]], se.fit=ste.fit[[nomeOK]])
         if (inherits(ogg, what = "glm", FALSE) && !link){
-            r[[2]] <- ogg$family$mu.eta(r[[1]])*r[[2]]
-            r[[1]] <- ogg$family$linkinv(r[[1]])
+          if(se.fit) r[[2]] <- ogg$family$mu.eta(r[[1]])*r[[2]]
+          r[[1]] <- ogg$family$linkinv(r[[1]])
             }
-        if(!se.fit) r<-r[1]
+        if(!se.fit) r<-r[1] #metti r[[1]] se vuoi fare restituire un vettore
         return(r)
         }
